from os.path import normpath, basename

from .http_client import ClientError
from .models.automodel import Automodel, AutomodelFeatures, AutomodelList
from .task import Task


class Automodels(object):
    """Automodel based API operations"""

    def __init__(self, base_client):
        self.client = base_client

    def get(self, automodel_id):
        automodel = self.client.request('GET', 'automodels/{}/'.format(automodel_id))
        return Automodel(automodel['automodelId'], automodel['datasetId'], automodel['name'], automodel['target'],
                         automodel['created'], automodel['recommendedModels'])

    def list(self, offset=0, limit=50):
        automodels = self.client.request('GET', 'automodels/', params={'offset': offset, 'limit': limit})
        print(automodels)
        return AutomodelList(automodels['data'], automodels['count'])

    def fit(self, dataset_id, target):
        response, status, headers = self.client.request_with_headers(
            'POST', 'automodels/', headers={'Accept': '*/*'}, data={
                'datasetId': dataset_id,
                'target': target
            })

        if status != 202:
            raise ClientError('{}/automodels/'.format(self.client.url), status, response)

        return Task(self.client, headers.get('location'), lambda result_url: self.get(basename(normpath(result_url))))

    def get_features(self, automodel_id):
        automodel_features = self.client.request('GET', 'automodels/{}/features/'.format(automodel_id))
        return AutomodelFeatures(automodel_features['automodelId'], automodel_features['target'],
                                        automodel_features['datasetId'], automodel_features['features'])
