from os.path import normpath, basename
from requests_toolbelt import MultipartEncoder
from requests_toolbelt.downloadutils import stream

from .http_client import ClientError
from .models.dataset import Dataset, DatasetList
from .task import Task


class Datasets(object):
    """Dataset based API operations"""

    def __init__(self, base_client):
        self.client = base_client

    def _import(self, source, content_type, data):
        response, status, headers = self.client.request_with_headers(
            'POST',
            'datasets/import/{}/'.format(source),
            headers={'Content-Type': content_type, 'Accept': '*/*'},
            data=data)

        if status != 202:
            raise ClientError('/datasets/import/{}/'.format(source), status, response)

        return Task(self.client, headers.get('location'), lambda result_url: self.get(basename(normpath(result_url))))

    def import_file(self, file_path):
        encoder = MultipartEncoder(fields={'file': (basename(file_path), open(file_path, 'rb'))})
        return self._import('file', encoder.content_type, encoder)

    def import_url(self, url):
        return self._import('url', 'application/json', {'url': url})

    def get(self, dataset_id):
        data = self.client.request('GET', 'datasets/{}'.format(dataset_id))
        return Dataset(data['id'], data['name'], data['created'])

    def download(self, dataset_id, file_path):
        r = self.client.request('GET', 'datasets/{}/download/'.format(dataset_id), raw_response=True, stream=True)
        filename = stream.stream_response_to_file(r, path=file_path)
        return filename

    def list(self, offset=0, limit=50):
        dataset_list = self.client.request('GET', 'datasets/', params={'offset': offset, 'limit':limit})
        return DatasetList(dataset_list['data'], dataset_list['count'])
