class Model(object):
    def __init__(self, badges, model_type, metrics):
        self._badges = badges
        self._model_type = model_type
        self._metrics = metrics

    @property
    def badges(self):
        return self._badges

    @property
    def model_type(self):
        return self._model_type

    @property
    def metrics(self):
        return self._metrics

    def __repr__(self):
        return 'Model({!r}, {!r}, {!r})'.format(self._badges, self._model_type, self._metrics)


class Automodel(object):
    def __init__(self, automodel_id, dataset_id, name, target, created, models):
        self._automodel_id = automodel_id
        self._dataset_id = dataset_id
        self._name = name
        self._target = target
        self._created_on = created
        self._models = [Model(m['modelBadge'], m['modelType'], m['metrics']) for m in models]

    @property
    def automodel_id(self):
        return self._automodel_id

    @property
    def dataset_id(self):
        return self._dataset_id

    @property
    def name(self):
        return self._name

    @property
    def target(self):
        return self._target

    @property
    def created_on(self):
        return self._created_on

    @property
    def models(self):
        return self._models

    def __repr__(self):
        return 'Automodel({!r}, {!r}, {!r}, {!r}, {!r}, {!r})'.format(
            self._automodel_id, self._dataset_id, self._name, self._target, self._created_on, self._models)


class Feature(object):
    def __init__(self, feature_type, name):
        self._type = feature_type
        self._name = name

    @property
    def feature_type(self):
        return self._type

    @property
    def name(self):
        return self._name

    def __repr__(self):
        return 'Feature({!r}, {!r})'.format(self._type, self._name)


class AutomodelFeatures(object):
    def __init__(self, automodel_id, target, dataset_id, feature_list):
        self._automodel_id = automodel_id
        self._target = target
        self._dataset_id = dataset_id
        self._feature_list = [Feature(f['type'], f['name']) for f in feature_list]

    @property
    def automodel_id(self):
        return self._automodel_id

    @property
    def target(self):
        return self._target

    @property
    def dataset_id(self):
        return self._dataset_id

    @property
    def feature_list(self):
        return self._feature_list

    def __repr__(self):
        return 'AutomodelFeatures({!r}, {!r}, {!r}, {!r})'.format(self._automodel_id, self._target,
                                                                  self._dataset_id, self._feature_list)


class AutomodelList(object):
    def __init__(self, automodels, count):
        self._automodels = [
            Automodel(s['automodelId'], s['datasetId'], s['name'], s['target'], s['created'],
                      s['recommendedModels']) for s in automodels
        ]
        self._count = count

    @property
    def automodels(self):
        return self._automodels

    @property
    def count(self):
        return self._count
