from os.path import normpath, basename
from requests_toolbelt import MultipartEncoder

from .http_client import ClientError
from .task import Task
from .models.prediction import PredictionList


class Predictions(object):
    def __init__(self, base_client):
        self.client = base_client

    def predict(self, automodel_id, test_data_file):
        encoder = MultipartEncoder(fields={'file': (basename(test_data_file), open(test_data_file, 'rb'))})
        response, status, headers = self.client.request_with_headers(
            'POST',
            'automodels/{}/predictions/'.format(automodel_id),
            headers={
                'Content-Type': encoder.content_type,
                'Accept': '*/*'
            },
            data=encoder)

        if status != 202:
            raise ClientError('automodels/{}/predictions/'.format(automodel_id), status, response)

        return Task(self.client, headers.get('location'),
                    lambda result_url: self.get(automodel_id, basename(normpath(result_url))))

    def get(self, automodel_id, prediction_id, offset=0, limit=50):
        prediction_data = self.client.request(
            'GET', 'automodels/{}/predictions/{}/'.format(automodel_id, prediction_id),
            params={'offset':offset, 'limit':limit})

        return PredictionList(prediction_data['id'], prediction_data['count'], prediction_data['data'])
